package com.mangst.sudoku;

/*
 * Copyright 2009 Mike Angstadt
 * mike.angstadt@gmail.com
 * http://www.mangst.com/
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.TreeSet;

import com.mangst.io.SpecialBufferedReader;

public class Board {
	/**
	 * Height/width of the board.
	 */
	public int size;
	
	/**
	 * Number of cells currently filled.
	 */ 
	public int filled;
	
	/**
	 * Number of iterations it took to solve the puzzle.
	 */
	public int iterations;

	/**
	 * The original state of the board.
	 */
	public Cell orig[][];
	
	/**
	 * The board state.
	 */
	public Cell cells[][];

	/**
	 * The 3x3 square areas.
	 */
	private Set<Cell> squares[];
	
	/**
	 * Each row.
	 */
	private Set<Cell> rows[];
	
	/**
	 * Each column.
	 */
	private Set<Cell> cols[];

	public Board(Reader reader) throws IOException, BoardFormatException {
		load(reader);
	}
	
	/**
	 * Loads the board with its initial state.
	 * @param reader
	 * @throws IOException
	 * @throws BoardFormatException
	 */
	public void load(Reader reader) throws IOException, BoardFormatException{
		BufferedReader in = null;
		try {
			in = new SpecialBufferedReader(reader);

			//get size of board
			try {
				size = Integer.parseInt(in.readLine());
				if (size <= 0) {
					throw new NumberFormatException();
				}
			} catch (NumberFormatException e) {
				throw new BoardFormatException("Board size not a valid number.");
			}

			//build set of all possible values to initialize each empty cell with
			Set<Integer> allPoss = new HashSet<Integer>();
			for (int i = 1; i <= size; ++i) {
				allPoss.add(i);
			}

			//initialize quick-lookup arrays
			rows = new Set[size];
			cols = new Set[size];
			squares = new Set[size];
			for (int i = 0; i < size; ++i) {
				rows[i] = new HashSet<Cell>();
				cols[i] = new HashSet<Cell>();
				squares[i] = new HashSet<Cell>();
			}

			//get board state
			filled = 0;
			cells = new Cell[size][size];
			orig = new Cell[size][size];
			for (int row = 0; row < size; row++) {
				String line = in.readLine();
				if (line == null) {
					throw new BoardFormatException("Board input incomplete.");
				}
				line = line.trim();

				String split[] = line.split("\\s+");
				if (split.length != size) {
					throw new BoardFormatException("Row " + (row + 1) + " of board input incorrect.");
				}
				for (int col = 0; col < size; ++col) {
					Cell c;
					try {
						int value = Integer.parseInt(split[col] + "");
						if (value <= 0 || value > size) {
							throw new BoardFormatException("Cell value bad: (" + row + "," + col + ") = " + value + ".");
						}
						c = new Cell(value);
					} catch (NumberFormatException e) {
						//if it can't parse the value, make the cell blank
						c = new Cell();
					}
					if (c.isEmpty()) {
						//only need to init the cell's possible values if it's empty
						c.poss = new HashSet<Integer>(allPoss);
					} else {
						filled++;
					}
					c.row = row;
					c.col = col;
					cells[row][col] = c;
					rows[row].add(c);
					cols[col].add(c);
					orig[row][col] = new Cell(c);
				}
			}

			//if square input is not provided, provide one
			String line = in.readLine();
			if (line == null) {
				double d = Math.sqrt(size);
				double floored = Math.floor(d);

				//if size is a perfect square, we can generate default square input
				if (d - floored < 0.0000001) {
					int squ = (int) d;
					StringBuilder sb = new StringBuilder();
					int num1 = 1;
					for (int i = 0; i < squ; ++i) {
						for (int j = 0; j < squ; ++j) {
							int num2 = num1;
							for (int k = 0; k < squ; ++k) {
								for (int l = 0; l < squ; ++l) {
									sb.append(num2 + " ");
								}
								num2++;
							}
							sb.append('\n');
						}
						num1 += squ;
					}
					in.close();
					in = new BufferedReader(new StringReader(sb.toString()));
					line = in.readLine();
				} else {
					throw new BoardFormatException("Must provide square input.");
				}
			}

			//figure out which cells belong to the 3x3 squares
			for (int row = 0; row < size; row++) {
				if (line == null) {
					throw new BoardFormatException("Incorrect square input.");
				}
				String split[] = line.split("\\s+");
				if (split.length != size) {
					throw new BoardFormatException("Row " + (row + 1) + " of square input incorrect.");
				}
				for (int col = 0; col < size; ++col) {
					int index;
					try {
						index = Integer.parseInt(split[col]);
						if (index <= 0 || index > size) {
							throw new NumberFormatException();
						}
						index--;
					} catch (NumberFormatException e) {
						throw new BoardFormatException("Square value bad: (" + row + "," + col + ") = " + split[col] + ".");
					}
					Cell c = getCell(row, col);
					c.square = index;
					squares[index].add(c);
				}
				line = in.readLine();
			}
			
			//verify board is valid (e.g. there aren't two "5"s in the same row)
			final Set<Cell> allAreaTypes[][] = new Set[][] { rows, cols, squares };
			for (Set<Cell> areaType[] : allAreaTypes){
				for (Set<Cell> area : areaType){
					Set<Integer> values = new HashSet<Integer>();
					for (Cell c : area){
						if (!c.isEmpty()){
							if (values.add(c.value) == false){
								//duplicate value, input is bad
								throw new BoardFormatException("Duplicate value at cell (" + c.row + "," + c.col + ").");
							}
						}
					}
				}
			}
			
			//verify square input is valid
			for (int i = 0; i < squares.length; ++i){
				Set<Cell> square = squares[i];
				if (square.size() != size){
					throw new BoardFormatException("Square input incorrect.  Too many " + (i+1) + "s.");
				}
			}
		} finally {
			if (in != null) {
				in.close();
			}
		}
	}

	public Cell getCell(int row, int col) {
		if (row < 0 || row >= size || col < 0 || col >= size) {
			throw new IndexOutOfBoundsException("Cell at (" + row + "," + col + ") does not exist.");
		}
		return cells[row][col];
	}

	@Override
	public String toString() {
		StringBuilder s = new StringBuilder();
		for (int i = 0; i < size; ++i) {
			s.append(getCell(i, 0));
			for (int j = 1; j < size; ++j) {
				s.append(' ');
				s.append(getCell(i, j));
			}
			s.append('\n');
		}
		return s.substring(0, s.length() - 1);
	}

	public String toStringHTML() {
		StringBuilder s = new StringBuilder();
		s.append("<html><head><style>td{height:" + (size * 4) + "px;width:" + (size * 4) + "x;}td th{height:" + (size * 2) + "px;width:" + (size * 2) + "px;font-size:0.8em;color:#999;}td td{height:" + (size * 2) + "px;width:" + (size * 2) + "px;}</style></head><body>");
		s.append("<table border='1'>");
		for (int row = 0; row < size; ++row) {
			s.append("<tr>");
			for (int col = 0; col < size; ++col) {
				Cell c = cells[row][col];
				if (c.isEmpty()) {
					s.append("<td>" + new TreeSet<Integer>(c.poss) + "</td>");
				} else {
					s.append("<td><span style='color:red;'>" + c + "</span></td>");
				}
			}
			s.append("</tr>");
		}
		s.append("</table>");
		s.append("</body></html>");
		return s.toString();
	}

	private int sameRow(Collection<Cell> col) {
		int index = -1;
		for (Cell c : col) {
			if (index < 0) {
				index = c.row;
			}
			if (c.row != index) {
				index = -1;
				break;
			}
		}
		return index;
	}

	private int sameCol(Collection<Cell> col) {
		int index = -1;
		for (Cell c : col) {
			if (index < 0) {
				index = c.col;
			}
			if (c.col != index) {
				index = -1;
				break;
			}
		}
		return index;
	}

	private int sameSquare(Collection<Cell> col) {
		int index = -1;
		for (Cell c : col) {
			if (index < 0) {
				index = c.square;
			}
			if (c.square != index) {
				index = -1;
				break;
			}
		}
		return index;
	}

	private void newSizePoss() {
		final Set<Cell> allAreaTypes[][] = new Set[][] { rows, cols, squares };
		for (Set<Cell> areaType[] : allAreaTypes) {
			for (Set<Cell> area : areaType) {

				//================================================
				//e.g. two cells are the only cells in their area to have poss values of 3,5...
				//...then remove 3,5 from all the areas that they have in common (at least 1 area, at most 2)
				
				//4
				//6
				//9
				//-
				//8
				//7
				//5
				//-----
				//2 . 5
				//. . 8
				//. . 6
				//cells at (6-8,1) cannot be 1 or 3 because cells at (7-8,0) *must* be either 1 or 3

				//key: poss.size(), value = cells whose poss.size()==key
				Map<Integer, List<Cell>> m = new HashMap<Integer, List<Cell>>();
				for (Cell c : area) {
					int p = c.poss.size();
					List<Cell> l = m.get(p);
					if (l == null) {
						l = new ArrayList<Cell>();
						m.put(p, l);
					}
					l.add(c);
				}
				for (int possSize = 2; possSize <= size; ++possSize) { //start at 2 because 0=only for initially filled cells, 1=is ready to be filled
					List<Cell> l = m.get(possSize);
					if (l != null && l.size() >= possSize) {
						List<Cell> matching = new ArrayList<Cell>();
						for (int j = 0; j < l.size(); ++j) {
							matching.add(l.get(j));
							for (int k = j + 1; k < l.size(); ++k) {
								if (l.get(j).poss.equals(l.get(k).poss)) {
									matching.add(l.get(k));
								}
							}
							if (matching.size() == possSize) {
								//e.g. two cells are the only cells in their area to have poss values of 3,5...
								//...then remove 3,5 from all the areas that they have in common (at least area 1, at most 2)

								//row
								{
									int row = sameRow(matching);
									if (row >= 0) {
										for (Cell c : rows[row]) {
											if (!matching.contains(c) && c.isEmpty()) {
												c.poss.removeAll(matching.get(0).poss);
											}
										}
									}
								}

								//col
								{
									int col = sameCol(matching);
									if (col >= 0) {
										for (Cell c : cols[col]) {
											if (!matching.contains(c) && c.isEmpty()) {
												c.poss.removeAll(matching.get(0).poss);
											}
										}
									}
								}

								//square
								{
									int square = sameSquare(matching);
									if (square >= 0) {
										for (Cell c : squares[square]) {
											if (!matching.contains(c) && c.isEmpty()) {
												c.poss.removeAll(matching.get(0).poss);
											}
										}
									}
								}
							}
							matching.clear();
						}
					}
				}

				//==========================================
				//e.g. if two cells are the only cells in an area to contain two possible values...
				//...then remove all other poss values from those cells and remove those two values for all other cells in the shared area(s)
				
				//init
				Set<Cell> a[] = new Set[size];
				for (int i = 0; i < a.length; ++i) {
					a[i] = new HashSet<Cell>();
				}

				//get poss value counts and Cells those poss values belong to
				//a[x] == cells that have a poss value of x
				for (Cell c : area) {
					if (c.isEmpty()) {
						for (int v : c.poss) {
							a[v - 1].add(c);
						}
					}
				}

				for (int i = 0; i < a.length; ++i) {
					if (a[i].size() == 1) {
						Cell c = a[i].iterator().next();
						c.poss.clear();
						c.poss.add(i + 1);
					} else if (a[i].size() == 2) {
						for (int j = i + 1; j < a.length; ++j) {
							if (a[j].size() == 2) {
								if (a[i].equals(a[j])) {
									//remove everything but the two values from the poss values of the two cells
									for (Cell c : a[i]) {
										c.poss.clear();
										c.poss.add(i + 1);
										c.poss.add(j + 1);
									}

									//find the areas they share (either 1 or 2 in total) and remove the two values from the poss values of each cell in each of those area
									//(if two areas are found, one of those areas will already be processed, since this is how we found these two cells in the first place, but it doesn't hurt to go thru them again)
									Cell pair[] = a[i].toArray(new Cell[] {});
									if (pair[0].row == pair[1].row) {
										//same row
										for (Cell c : rows[pair[0].row]) {
											if (c != pair[0] && c != pair[1] && c.isEmpty()) {
												c.poss.remove(i + 1);
												c.poss.remove(j + 1);
											}
										}
									}
									if (pair[0].col == pair[1].col) {
										//same col
										for (Cell c : cols[pair[0].col]) {
											if (c != pair[0] && c != pair[1] && c.isEmpty()) {
												c.poss.remove(i + 1);
												c.poss.remove(j + 1);
											}
										}
									}
									if (pair[0].square == pair[1].square) {
										//same square
										for (Cell c : squares[pair[0].square]) {
											if (c != pair[0] && c != pair[1] && c.isEmpty()) {
												c.poss.remove(i + 1);
												c.poss.remove(j + 1);
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}

	public void solve() {
		iterations = 0;

		final Set<Cell> ROWS_COLS_SQUARES[][] = new Set[][] { rows, cols, squares };
		final Set<Cell> ROWS_COLS[][] = new Set[][] { rows, cols };
		
		//init poss values for each cell
		for (Cell row[] : cells) {
			for (Cell c : row) {
				if (c.isEmpty()) {
					Set<Cell> areas[] = new Set[] { rows[c.row], cols[c.col], squares[c.square] };
					for (Set<Cell> area : areas) {
						for (Cell innerC : area) {
							if (!innerC.isEmpty()) {
								c.poss.remove(innerC.value);
							}
						}
					}
				}
			}
		}

		boolean valueFound = true;
		while (valueFound) { //loop until it doesn't fill any new cells
			newSizePoss();

			//. . .
			//. 8 2
			//. . .
			//-----|-----|-----
			//9 7 .|. . 3|6 . .
			//. . 5|2 . .|. . .
			//. . .|. 7 1|. . .
			//cells (5,6-8) cannot be 2 because one of cells at (5,0-1) *has* to be a 2
			for (Set<Cell> square : squares) {
				for (Set<Cell> s[] : ROWS_COLS) {
					for (Set<Cell> row : s) {
						Set<Cell> intersection = new HashSet<Cell>(square);
						intersection.retainAll(row); //{1,2,3}.retainAll({3,4,5}) == {3}
						if (intersection.size() > 0) { //should always == 3 in 9x9 game
							Set<Cell> inSquareNotRow = new HashSet<Cell>(square);
							inSquareNotRow.removeAll(row); // {1,2,3}.removeAll({3,4,5}) == {1,2}

							Set<Integer> valuesInIntersection = new HashSet<Integer>();
							for (Cell c : intersection) {
								if (c.isEmpty()) {
									valuesInIntersection.addAll(c.poss);
								}
							}
							Set<Integer> valuesInNonRowArea = new HashSet<Integer>();
							for (Cell c : inSquareNotRow) {
								if (c.isEmpty()) {
									valuesInNonRowArea.addAll(c.poss);
								}
							}

							valuesInIntersection.removeAll(valuesInNonRowArea);
							for (int value : valuesInIntersection) {
								for (Cell c : row) {
									if (!intersection.contains(c) && c.isEmpty()) {
										c.poss.remove(value);
									}
								}
							}
						}
					}
				}
			}

			//fill all cells with only one poss value
			valueFound = false;
			Stack<Cell> filledCells = new Stack<Cell>();
			for (Cell row[] : cells) {
				for (Cell c : row) {
					if (c.isEmpty()) {
						if (c.poss.size() == 1) {
							c.value = c.poss.iterator().next();
							valueFound = true;
							filledCells.push(c);
							filled++;
						}
					}
				}
			}
			
			//update poss values of each cell who is in the area of a filled cell
			while (!filledCells.empty()){
				Cell c = filledCells.pop();
				for (Cell rowC : rows[c.row]){
					if (rowC.isEmpty()){
						rowC.poss.remove(c.value);
						if (rowC.poss.size() == 1){
							//fill it
							rowC.value = rowC.poss.iterator().next();
							filledCells.push(rowC);
							filled++;
						}
					}
				}
				for (Cell colC : cols[c.col]){
					if (colC.isEmpty()){
						colC.poss.remove(c.value);
						if (colC.poss.size() == 1){
							//fill it
							colC.value = colC.poss.iterator().next();
							filledCells.push(colC);
							filled++;
						}
					}
				}
				for (Cell squareC : squares[c.square]){
					if (squareC.isEmpty()){
						squareC.poss.remove(c.value);
						if (squareC.poss.size() == 1){
							//fill it
							squareC.value = squareC.poss.iterator().next();
							filledCells.push(squareC);
							filled++;
						}
					}
				}
			}

			iterations++;
		}
	}
}
